package edu.uiuc.ncsa.security.oauth_1_0a;

import edu.uiuc.ncsa.security.core.exceptions.GeneralException;
import edu.uiuc.ncsa.security.core.exceptions.ServerSideException;
import edu.uiuc.ncsa.security.core.util.MapUtilities;
import edu.uiuc.ncsa.security.delegation.client.request.AccessTokenRequest;
import edu.uiuc.ncsa.security.delegation.client.request.AccessTokenResponse;
import edu.uiuc.ncsa.security.delegation.client.request.AuthorizationGrantRequest;
import edu.uiuc.ncsa.security.delegation.client.request.AuthorizationGrantResponse;
import edu.uiuc.ncsa.security.delegation.client.server.AuthorizationServer;
import edu.uiuc.ncsa.security.delegation.services.Request;
import edu.uiuc.ncsa.security.delegation.services.Response;
import edu.uiuc.ncsa.security.oauth_1_0a.client.OAClient;
import net.oauth.OAuth;
import net.oauth.OAuthAccessor;
import net.oauth.OAuthMessage;
import net.oauth.OAuthProblemException;
import net.oauth.client.OAuthClient;
import org.apache.commons.codec.binary.Base64;

import javax.net.ssl.SSLPeerUnverifiedException;
import java.net.URI;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import static edu.uiuc.ncsa.security.oauth_1_0a.OAuthUtilities.whittleParameters;
import static net.oauth.OAuth.*;

/**
 * <p>Created by Jeff Gaynor<br>
 * on Mar 16, 2011 at  2:46:28 PM
 */
public class AuthorizationServerImpl extends AuthorizationServer {
    public static final String DEBUG_STACKTRACE_KEY = "stacktrace";

    public AuthorizationServerImpl(URI address) {
        super(address);
    }


    public Response process(Request request) {
        return request.process(this);
    }

    @Override
    public AuthorizationGrantResponse processAGRequest(AuthorizationGrantRequest agRequest) {
        return getAuthorizationGrant(agRequest);
    }

    @Override
    public AccessTokenResponse processATRequest(AccessTokenRequest atRequest) {
        return getAccessToken(atRequest);
    }

    protected AccessTokenResponse getAccessToken(AccessTokenRequest atRequest) {
        AccessTokenImpl accessToken = null;
        AuthorizationGrantImpl ag = null;
        VerifierImpl vImpl = null;

        OAClient oaClient = (OAClient) atRequest.getClient();
        OAuthAccessor accessor = OAuthUtilities.createOAuthAccessor(this, oaClient);

        if (atRequest.getAuthorizationGrant() instanceof AuthorizationGrantImpl) {
            ag = (AuthorizationGrantImpl) atRequest.getAuthorizationGrant();
        } else {
            // This should never happen, but is here since *someday* the codebase may change and this will intercept
            // such an error.
            throw new GeneralException("Internal Error: Incorrect authorization grant found. Should have been a TempCred but was a " + atRequest.getAuthorizationGrant().getClass());
        }

        if (atRequest.getVerifier() == null) {
            throw new GeneralException("Error: No verifier found. This is required by the OAuth spec.");
        }
        if (atRequest.getVerifier() instanceof VerifierImpl) {
            vImpl = (VerifierImpl) atRequest.getVerifier();
        } else {
            throw new GeneralException("Internal Error: Incorrect verifier instance found. Should have been a VerifierImpl but was a " + atRequest.getVerifier());

        }
        // Note that the OAuth 1.0a spec needs the verifier (set as an extra parameter)
        // and Authz grant shared secret (set in the accessor as per below), not the accessor.requestToken
        // which would send along a copy of the authz grant. The OAuth library should not be so simple-minded,
        //
        if (ag.getSharedSecret() != null) {
            accessor.tokenSecret = ag.getSharedSecret().toString();
        }

        if (oaClient.getSignatureMethod().equals(RSA_SHA1)) {
            accessor.setProperty(OAuthConstants.PRIVATE_KEY, oaClient.getSecret());
            accessor.consumer.setProperty(OAuthConstants.PRIVATE_KEY, oaClient.getSecret());
        }

        try {
            net.oauth.client.OAuthClient oauthClient = OAuthUtilities.newOAuthClient();
            ArrayList<String> arrayList = new ArrayList<String>();
            arrayList.add(OAuth.OAUTH_VERIFIER);
            arrayList.add(vImpl.getURIToken().toString());
            for (String key : atRequest.getParameters().keySet()) {
                arrayList.add(key);
                arrayList.add(atRequest.getParameters().get(key));
            }
            OAuthMessage message = oauthClient.getAccessToken(accessor, "GET", OAuth.newList(arrayList.toArray(new String[arrayList.size()])));
            HashMap m = whittleParameters(message);
            accessToken = new AccessTokenImpl(URI.create(message.getParameter(OAUTH_TOKEN)), URI.create(message.getParameter(OAUTH_TOKEN_SECRET)));
            AccessTokenResponse atr = new AccessTokenResponse(accessToken);
            atr.setParameters(m);
            return atr;
        } catch (Exception e) {
            handleException(e);
            return null;
/*

            e.printStackTrace();
            throw new GeneralException("Error accessing authorization server", e);
*/
        }
    }


    protected AuthorizationGrantResponse getAuthorizationGrant(AuthorizationGrantRequest agRequest) {
        List<Map.Entry<String, String>> params = MapUtilities.toList(agRequest.getParameters());
        OAClient oac = (OAClient) agRequest.getClient();
        OAuthAccessor accessor = OAuthUtilities.createOAuthAccessor(this, oac);
        if (oac.getSignatureMethod().equals(RSA_SHA1)) {
            accessor.consumer.setProperty(OAuthConstants.PRIVATE_KEY, oac.getSecret());
            accessor.setProperty(OAuthConstants.PRIVATE_KEY, oac.getSecret());
        }

        try {
            OAuthClient client = OAuthUtilities.newOAuthClient();
            OAuthMessage message = client.getRequestTokenResponse(accessor, "GET", params);
            String rt = message.getParameter(OAUTH_TOKEN);
            if (rt == null || rt.length() == 0) {
                throw new IllegalArgumentException("Error: delegation server did not return a request token");
            }
            String rtss = message.getParameter(OAUTH_TOKEN_SECRET);
            if (!((OAClient) agRequest.getClient()).getSignatureMethod().equals(RSA_SHA1) && (rtss == null || rtss.length() == 0)) {
                throw new IllegalArgumentException("Error: delegation server did not return a shared secret");
            }
            AuthorizationGrantImpl agi = new AuthorizationGrantImpl(URI.create(rt), URI.create(rtss));
            AuthorizationGrantResponse agr = new AuthorizationGrantResponse(agi);
            HashMap m = whittleParameters(message);
            // grab any unused parameters from the server, as per OAuth spec.
            agr.setParameters(m);
            return agr;
        } catch (Throwable e) {
            handleException(e);
            return null;
        }
    }

    protected void handleException(Throwable t) {
        if (t instanceof RuntimeException) {
            throw (RuntimeException) t;
        }

        if (t instanceof OAuthProblemException) {
            // If debugging is enabled, any server-side exceptions will be
            // returned in a B64 encoding. OAuth does support this as part of
            // its mechanism for errors, but getting the information from the
            // generic OAuthProblemException is messy (done here).
            OAuthProblemException opx = (OAuthProblemException) t;
            // take the exception. If there is a Location parameter, i.e., redirect,
            // parse that into key value pairs. Return a server side exception
            // Otherwise, just have generic error handling.
            ServerSideException sse = new ServerSideException(t);

            // A specific parameter in the OAuth exception that tells of a redirect to an
            // error location on the server. This is optional.
            if (opx.getParameters().get("Location") != null) {
                // ***IF*** the server supplies a diagnostic redirect, parse it
                // and set it as the redirect for the server side exception to read off
                // and handle.
                // OA4MP servers might also return a stack trace, which is a b64 encoded
                // stack trace from the server. The client must request this with
                // a parameter in the original call.
                String redirect = opx.getParameters().get("Location").toString();
                URI r = URI.create(redirect);
                String query = r.getQuery();
                // have to parse this since Java doesn't have a built in parser
                HashMap<String, String> qparams = new HashMap<String, String>();
                String[] arrParameters = query.split("&");
                for (String tempParameterString : arrParameters) {
                    String[] arrTempParameter = tempParameterString.split("=");
                    if (arrTempParameter.length >= 2) {
                        String parameterKey = arrTempParameter[0];
                        String parameterValue = arrTempParameter[1];
                        if (parameterKey.toLowerCase().equals(DEBUG_STACKTRACE_KEY)) {
                            try {
                                parameterValue = new String(Base64.decodeBase64(parameterValue));
                            } catch (Throwable x) {
                                System.out.println("Could not decode stack trace for cause " + x.getClass().getName() + " msg=\"" + x.getMessage() + "\", trace:" + parameterValue);
                                parameterValue = "(none)";
                            }
                        }
                        qparams.put(parameterKey, parameterValue);
                    }
                }
                sse.setQueryParameters(qparams);
                sse.setRedirect(r);
            }

            throw sse;
        }
        // so this is not an OAuth-intercepted exception. Something else happened.
        if (t.getCause() != null) {
            t.getCause().printStackTrace();
        } else {
            t.printStackTrace();
        }
                /*
                Special case this exception since a bad SSL configuration on the client side will
                most likely fail here first. This way they don't lose the exception and probable
                cause buried in a stack trace.
                 */
        if (t instanceof SSLPeerUnverifiedException) {
            throw new GeneralException("Error: could not connect to the server. Is your trusted roots store up to date?", t);
        }
        throw new GeneralException("Error invoking OAuth client", t);
    }

    @Override
    public String toString() {
        return "AuthorizationServerImpl[address=" + getAddress() + "]";
    }
}
